@@ -4,7 +4,9 @@ import logging
 from datetime import datetime
 from typing import Dict, Union
 from uuid import uuid4
+import urllib3
 
+import cyjax
 import pymisp
 from pymisp import MISPEvent, MISPAttribute, MISPOrganisation
 
@@ -35,13 +37,15 @@ indicator_map = {
 class Client:  # pylint: disable=too-few-public-methods disable=E1136
     """MISP client to add indicators."""
 
-    def __init__(self, misp_url: str, misp_key: str, debug: bool = False):
+    def __init__(self, misp_url: str, misp_key: str, ssl: bool = True, debug: bool = False):
         if debug:
             # Silent logs from pymisp
             logging.getLogger('pymisp').setLevel(logging.DEBUG)
+        if not ssl:
+            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
         self.added_events: Dict[str, str] = {}
         try:
-            self.misp = pymisp.PyMISP(misp_url, misp_key)
+            self.misp = pymisp.PyMISP(misp_url, misp_key, ssl=ssl)
         except pymisp.PyMISPError as misp_error:
             raise MispException(
                 'Unable to connect to MISP. Please make sure the URL and the API key are correct.') from misp_error
@@ -76,6 +80,32 @@ class Client:  # pylint: disable=too-few-public-methods disable=E1136
                             timestamp=datetime.strptime(indicator['discovered_at'], '%Y-%m-%dT%H:%M:%S%z'))
         self.misp.add_attribute(misp_event_uuid, attribute)
 
+    def publish_events(self) -> int:
+        """
+        Publishes all last added MISP events.
+        """
+        published_events = 0
+        for event_uuid in self.added_events.values():
+            try:
+                misp_event = self.misp.get_event(event_uuid)
+
+                if isinstance(misp_event, dict):
+                    is_published = misp_event.get('published', False)
+                else:
+                    is_published = getattr(misp_event, 'published', False)
+
+                if is_published:
+                    log.debug('Event "%s" already published', event_uuid)
+                    continue
+
+                log.debug('Publishing event "%s"', event_uuid)
+                self.misp.publish(misp_event)
+                published_events += 1
+            except pymisp.PyMISPError as misp_error:
+                log.error('Unable to publish event %s: %s', event_uuid, misp_error)
+                continue
+        return published_events
+
     def _search_event_by_uuid(self, event_uuid: str) -> Union[str, None]:
         """
         Searches a MISP event by UUID.
@@ -121,11 +151,35 @@ class Client:  # pylint: disable=too-few-public-methods disable=E1136
             misp_event = MISPEvent()
             misp_event.uuid = event_uuid
             misp_event.Orgc = misp_org
-            misp_event.from_dict(info=indicator['description'], date=indicator['discovered_at'], analysis=2)
+
+            misp_event.from_dict(info=indicator['description'],
+                                 date=indicator['discovered_at'],
+                                 analysis=2)
 
             if 'handling_condition' in indicator:
                 misp_event.add_tag("tlp:{}".format(indicator['handling_condition'].lower()))
 
+            # get mitre-attack linked to this indicator source (incident report)
+            try:
+                incident_report = cyjax.IncidentReport().one(int(source_id))
+            except cyjax.exceptions.ForbiddenException:
+                # this API key does not have access to incident report resource
+                incident_report = None
+
+            if incident_report:
+                # mitre techniques
+                if incident_report.get('techniques'):
+                    for i, technique_name in enumerate(incident_report.get('techniques')):
+                        technique_id = incident_report.get('technique_ids')[i]
+                        misp_event.add_tag('misp-galaxy:mitre-attack-pattern="{} - {}"'.format(technique_name,
+                                                                                               technique_id))
+
+                # mitre software
+                if incident_report.get('software'):
+                    for i, software_name in enumerate(incident_report.get('software')):
+                        software_id = incident_report.get('software_ids')[i]
+                        misp_event.add_tag('misp-galaxy:mitre-malware="{} - {}"'.format(software_name, software_id))
+
             self.misp.add_event(misp_event)
 
         # Cache the event UUID
